/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "PluginAnalytics.h"
#include "MixCastLog.h"
#include "Utility/MiscUtil.hpp"

#define ANALYTICS_CONFIG_SECTION TEXT("MixCast")
#define ANALYTICS_CONFIG_CHECK_KEY TEXT("AnalyticsDate")

void FPluginAnalytics::SendReport()
{
#if UE_EDITOR
	FString Date = FDateTime::Today().ToString().LeftChop(9);
	FString LastDate;
	GConfig->GetString(ANALYTICS_CONFIG_SECTION, ANALYTICS_CONFIG_CHECK_KEY, LastDate, GEditorIni);

	if (Date.Compare(LastDate) == 0)
	{
		return;
	}

	GConfig->SetString(ANALYTICS_CONFIG_SECTION, ANALYTICS_CONFIG_CHECK_KEY, *Date, GEditorIni);

	const FString RequestString = BuildRequestString();
	auto HttpRequest = FHttpModule::Get().CreateRequest();
	HttpRequest->SetVerb("POST");
	HttpRequest->SetHeader("Content-Type", "application/x-www-form-urlencoded");
	HttpRequest->SetURL(TEXT("https://analytics.blueprintreality.com/submit-unreal.php"));
	HttpRequest->SetContentAsString(RequestString);
	HttpRequest->OnProcessRequestComplete().BindStatic(&FPluginAnalytics::AnalyticsResponse);
	HttpRequest->ProcessRequest();
#endif
}

void FPluginAnalytics::AnalyticsResponse(const FHttpRequestPtr Request, const FHttpResponsePtr Response, const bool bWasSuccessful)
{
	if (Request->GetStatus() == EHttpRequestStatus::Failed_ConnectionError)
	{
		//Fail silently if server missing
		return;
	}

	if (!Response.IsValid())
	{
		MIXCAST_ERROR("Analytics request failed: no response");
		return;
	}

	if (!bWasSuccessful)
	{
		const FString responseStr = Response->GetContentAsString();
		MIXCAST_ERROR("Analytics request failed: %s", *responseStr);
		return;
	}
}

FString FPluginAnalytics::BuildRequestString()
{
	const FString CompanyName = MiscUtil::GetCompanyName();
	const FString DeviceId = MiscUtil::GetDeviceId();
	const FString ProjectName = MiscUtil::GetProjectName();
	return FString::Printf(TEXT("orgName=%s&uid=%s&projectName=%s&mixcastSDK=%s&unrealVer=%s"), *CompanyName, *DeviceId, *ProjectName, *SDK_VERSION, *UNREAL_VERSION);
}
