/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#define TICKS_PER_SECOND (1000LL * 10000)

#include "CoreMinimal.h"
#include "Engine/World.h"
#include "Kismet/GameplayStatics.h"
#include "Utility/MiscUtil.hpp"

class MIXCAST_API FFrameTimer
{
public:
	static FString GetSMPTE(unsigned long timestamp, float framerate)
	{
		unsigned long hoursPart = timestamp / (unsigned long)(framerate * 60 * 60);
		unsigned long minutesPart = (timestamp / (unsigned long)(framerate * 60)) % 60;
		unsigned long secondsPart = (timestamp / (unsigned long)framerate) % 60;
		unsigned long framePart = timestamp % (unsigned long)framerate;
		return FString::Printf(TEXT("%02d:%02d:%02d:%02d"), (int)hoursPart, (int)minutesPart, (int)secondsPart, (int)framePart); //assumes framerate is less than 100
	}

	float GetFrameRate() { return frameRate; }

	bool IsStarted() { return isStarted; }

	unsigned long long LastUpdateTime() { return lastUpdateTime; }
	unsigned long long LastIdealTime() { return lastIdealTime; }

	unsigned long LastFrameIndex() { return lastFrameIndex; }

	FFrameTimer()
	{
		isStarted = false;
	}

	void Start(float framesPerSecond)
	{
		frameRate = framesPerSecond;

		startTime = MiscUtil::GetMixCastTimestamp();

		lastUpdateTime = startTime;
		lastFrameIndex = 0;
		lastIdealTime = startTime;

		isStarted = true;
	}
	void Update()
	{
		lastUpdateTime = MiscUtil::GetMixCastTimestamp();
		lastFrameIndex = GetFrameIndexForTimestamp(lastUpdateTime - startTime, frameRate);
		lastIdealTime = GetIdealTimestampForFrameIndex(lastFrameIndex, frameRate) + startTime;
	}

	static unsigned long GetFrameIndexForTimestamp(unsigned long long durationTicks, float frameRate)
	{
		unsigned long long extraDuration = (unsigned long long)(TICKS_PER_SECOND / 2 / frameRate);
		return (unsigned long)((double)frameRate * (durationTicks + extraDuration) / TICKS_PER_SECOND);
	}
	static unsigned long long GetIdealTimestampForFrameIndex(unsigned long frameIndex, float frameRate)
	{
		return (unsigned long long)((double)frameIndex * TICKS_PER_SECOND / frameRate);
	}

	static unsigned long GetMidnightRelativeFrameIndex(unsigned long long timestamp, float frameRate)
	{
		const unsigned long long TicksPerDay = 24LL * 60 * 60 * 1000 * 10000;
		unsigned long long ticksPerFrame = (unsigned long long)(TICKS_PER_SECOND / (double)frameRate);
		return GetFrameIndexForTimestamp((timestamp - timestamp % ticksPerFrame) % TicksPerDay, frameRate);
	}

private:
	bool isStarted;

	unsigned long long lastUpdateTime;
	unsigned long long lastIdealTime;
	unsigned long lastFrameIndex;

	float frameRate;
	unsigned long long  startTime;
};
