/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"

class MIXCAST_API MathUtil
{
public:
	static FVector UnityPositionToUnreal(const FVector &UnityPosition)
	{
		// Unity uses X for left, Y for up and Z for forward, but Unreal uses X for forward, Y for left and Z for up.
		// Both coordinate systems are left handed. Unreal also uses centimeters for units instead of meters.
		return {
			UnityPosition.Z * 100.0f, // X
			UnityPosition.X * 100.0f, // Y
			UnityPosition.Y * 100.0f, // Z
		};
	}
	static FVector UnrealPositionToUnity(const FVector &UnrealPosition)
	{
		return {
			UnrealPosition.Y * 0.01f,
			UnrealPosition.Z * 0.01f,
			UnrealPosition.X * 0.01f,
		};
	}

	static FVector UnityVectorToUnreal(const FVector &UnityVec)
	{
		return {
			UnityVec.Z, // X
			UnityVec.X, // Y
			UnityVec.Y, // Z
		};
	}

	static FQuat UnityRotationToUnreal(const FQuat &UnityRotation)
	{
		return {
			UnityRotation.Z,
			UnityRotation.X,
			UnityRotation.Y,
			UnityRotation.W,
		};
	}
	static FQuat UnrealRotationToUnity(const FQuat &UnrealRotation)
	{
		return {
			UnrealRotation.Y,
			UnrealRotation.Z,
			UnrealRotation.X,
			UnrealRotation.W
		};
	}

	//degrees in/out
	static float VerticalFoVToHorizontal(const float vertFoV, const float aspectRatio)
	{
		const float vFovRadians = vertFoV * PI / 180.0f;
		const float hFovRadians = atan(tan(vFovRadians / 2.0f) * aspectRatio) * 2.0f;
		return hFovRadians / PI * 180.0f;
	}
};
